@@ -1,8 +1,9 @@
 package Lingua::PT::PLN;
 use strict;
 
+
 use Lingua::PT::PLNbase;
-use Lingua::PT::PLN::Words2Sampa;
+#use Lingua::PT::PLN::Words2Sampa;
 
 require Exporter;
 our @ISA = qw(Exporter AutoLoader);
@@ -11,7 +12,7 @@ our @EXPORT =
   (@Lingua::PT::PLNbase::EXPORT,
    qw(syllable accent wordaccent oco initPhon toPhon));
 
-our $VERSION = '0.21';
+our $VERSION = '0.22';
 
 use POSIX qw(locale_h);
 setlocale(&POSIX::LC_ALL, "pt_PT");
@@ -117,6 +118,7 @@ sub oco {
 	print SORT  "$i $oco{$i}";
       }
     } else {
+      binmode STDOUT, ":utf8" if $opt{encoding};
       for my $i (sort keys %oco ) {
 	print  "$i $oco{$i}";
       }
@@ -141,12 +143,14 @@ sub wordaccent {
   for ($p) {
     s{(\w*$acento)}{"$1}i             or  # word with an accent character
 #      s{(\w*)([ua])(ir)$}{$1$2|"$3}i  or  # word ending with air uir
-      s{(\w*([zlr]|[iu]s?))$}{"$1}i   or  # word ending with z l r i u is us
+      s{(\w*([zlr]|[iu]s?|um))$}{"$1}i   or  # word ending with z l r i u is us
       s{(\w+\|\w+)$}{"$1}             or  # accent in 2 syllable frm the end
       s{(\w)}{"$1};                       # accent in the only syllable
 
     if(!$flag){
-      s{"(([qg]u|$consoante)*($vogal|[yw]))}{$1:}i; # accent in the 1.st vowel
+      # s{"(([qg]u|$consoante)*($vogal|[yw]))}{$1:}i; # accent in the 1.st vowel
+      # bugfix: quo|ti|"dia|no => quo|ti|dia:|no; op|"o => op|:o
+      s{"(([qg]u|$consoante)*([i]$vogal|$vogal|[yw]))}{$1:}i; # accent in the 1.st vowel
       s{:($acento)}{$1:}i;                         # mv accent after accents
       s{"}{}g;
     }
@@ -189,18 +193,122 @@ sub syllable{
        else{$1}
       }ge;
 
-    s{([a])(i[ru])}{$1|$2}i;		#ditongos and friends
-    s{([ioe])([aoe])}{$1|$2}ig;
-    s{u(ai|ou|a)}{u|$1}i;
-    s{([^qg]u)(ei|iu|ir|$acento|e)}{$1|$2}i;	# continu|e
-    s{([^q]u)(o)}{$1|$2}i;			# quo|ta; vcu|o
+    s{([a])(i[ru])$}{$1|$2}i;   #ditongos and friends
+
+    #s{([ioe])([aoe])}{$1|$2}ig; # not this simple
+    s{(?<!^h)([ioe])([e])}{$1|$2}ig;   #
+    s{([ioe])([ao])}{$1|$2}ig;    # la|go|a; di|a; ru|b|o|la; n|on; au|r|o|la
+
+    s{([^qg]u)(ai|ou|a)}{$1|$2}i;   # BUGFIX:  [^q]: qu|a|se => qua|se
+    s{([^qgc]u)(i|ei|iu|ir|$acento|e)}{$1|$2}i; # continu|e
+    s{([lpt]u)\|(i)(?=\|[ao])}{$1$2}ig;  # a|le|lui|a ta|pui|a con|lui|o
+    s{([^q]u)(o)}{$1|$2}i;      # quo|ta; vcu|o
     s{([aeio])($acento)}{$1|$2}i;
     s{([])($vogal)}{$1|$2}i;
     s{^a(o|e)}{a|$1}i;			# a|onde; a|orta; a|erdromo
 
-    s{([qg]u)\|([ei])}{$1$2}i;
+    # Exemplo: his|t|ria
+    if( m/([\w\|]*$acento[\w\|]*)\-/) {
+      if ( m/([eiou])\|([aeio])\-/ ) { # palavras com hifen
+        s{([eiou])\|([aeio])(?=\-)}{$1$2}ig if $1 ne $2 ; 
+      }
+    }
+    if ( m/([\w\|]*$acento[\w\|]*)$/ ) {
+      if ( m/([eiou])\|([aeio])$/ ) {
+        s{([eiou])\|([aeio])$}{$1$2}ig if $1 ne $2 ;
+      }
+    }
+
+    #s{([qg]u)\|([ei])}{$1$2}i;
+
+    s{rein}{re|in}ig;  # re|in|ves|tir
+    s{ae}{a|e}ig; # ma|ca|en|se ; es|ma|e|cer
+    s{ain}{a|in}ig;  # a|in|da con|tra|in|for|mar pa|in|o
+    s{ao(?!s)}{a|o}ig; # ca|o|ti|ca|men|te ; ma|o|me|tis|mo != caos
+    s{cue}{cu|e}ig; # ar|cu|en|se
+    s{cui(?=\|?[mnr])}{cu|i}ig; #
+    s{cui(?=\|da\|de$)}{cu|i}ig; # pro|mis|cu|i|da|de; i|no|cu|i|da|de
+    s{coi(?=[mn])}{co|i}ig; # co|in|ci|den|te;
+    s{cai(?=\|?[mnd])}{ca|i}ig; # des|ca|i|de|la; des|ca|i|men|to
+    s{ca\|i(?=\|?[m]$acento)}{cai}ig; # cai|mo
+    s{cu([])}{cu|$1}ig; # pe|cu||ria va|cu||me|tro
+    s{ai(?=\|?[z])}{a|i}ig; # ra|iz ; en|ra|i|zar
+    s{i(u\|?)n}{i|$1n}ig; # tri|un|fo
+    s{i(u\|?)r}{i|$1r}ig; # di|ur|no
+    s{i(u\|?)v}{i|$1v}ig; # viuvar
+    s{i(u\|?)l}{i|$1l}ig; # friulano
+    s{ium}{i|um}ig; # mdium
+    s{([ta])iu}{$1i|u}ig; # multiuso , baiuca,  maiusculizar
+    s{miu\|d}{mi|u|d}ig; # miudinho
+    s{au\|to(?=i)}{au|to|}ig; # au|to|i|ma|gem
+    s{(?<=$vogal)i\|nh(?=[ao])}{|i|nh}ig; # # ven|to|i|nha
+    s{oi([mn])}{o|i$1}ig; # a|men|do|im monoinsaturado microinjeco
+    s{oi\|b}{o|i|b}ig; # proibido
+    s{ois(?!$)}{o|is}ig; # ma|o|is|mo ; e|go|is|ti|ca|men|te
+    s{o(i\|?)s(?=$acento)}{o|$1s}ig; # ra|di|o|i|s|to|po
+    s{([dtm])aoi}{$1a|o|i}ig; # da|o|is|ta ; ta|o|is|ta ; ma|o|is|ta
+    s{(?<=[trm])u\|i(?=\|?[tvb][oa])}{ui}ig;
+    # ?: gra|tui|ti|da|de vs in|tu|i|ti|vo
+
+    s{^gas\|tro(?!-)}{gas|tro|}ig; # gas|tro|in|tes|ti|nal ; gas|tro-in|tes|ti|nal
+    s{^fais}{fa|is}ig; # faiscar | faiscao
+    s{^hie}{hi|e}ig; # hi|e|na ; hi|e|rar|ca
+    s{^ciu}{ci|u}ig; # ci|u|men|to
+    s{(?<=^al\|ca)\|i}{i}ig; # al|cai|de
+    s{(?<=^an\|ti)(p)\|?}{|$1}ig; # anti
+    s{(?<=^an\|ti)(\-p)\|?}{$1}ig; # anti
+    s{(?<=^neu\|ro)p\|}{|p}ig; # neu|ro|psi|c|lo|go
+    s{(?<=^pa\|ra)p\|}{|p}ig; # pa|ra|psi|c|lo|go
+    s{(?<=^ne\|)op\|}{o|p}ig; # neopsicadlico
+    s{^re(?=[i]\|?[md])}{re|}ig; # re|i|dra|ta|o
+    s{^re(?=i\|n[i]\|c)}{re|}ig; # re|i|ni|ci|ar
+    s{^re(?=i\|nau\|g)}{re|}ig; # re|i|nau|gu|rar
+    s{^re(?=[u]\|?[ntsr])}{re|}ig; # re|u|so
+    s{(?<=^vi\|de\|)o($vogal)}{o|$1}ig; # vi|de|o|a|ma|dor
+
+    s{^su\|b(?!$vogal)}{sub|}ig;
+    s{(?<=[\|\-])su\|b(?!$vogal)}{sub|}ig;
+    s{^sub\|l(?=i\|?m)}{su|bl}ig; # su|bli|mar (excepo, compostas: sub|li|mi|nar sub|li|mi|te)
+    s{(?<=\|)sub\|l(?=i\|?m)}{su|bl}ig;
+    s{^sub\|l(?=i\|?nh)}{su|bl}ig;  # (excepo: sublinha)
+    s{(?<=\|)sub\|l(?=i\|?nh)}{su|bl}ig;
+    s{^sub\|s(?=\|?$vogal)}{sub|s}ig;
+    s{(?<=\|)sub\|s(?=\|?$vogal)}{sub|s}ig;
+    s{^sub\|s(?=\|?$consoante)}{subs|}ig;
+    s{(?<=\|)sub\|s(?=\|?$consoante)}{subs|}ig;
+
+    s{so\|bs}{sobs}ig; # de|sobs|tru|ir
+
+    s{(?<=\|)trai(?=\|d)}{tra|i}ig;  # re|tra|i|da|men|te ; des|con|tra|i|da|men|te
+
+    s{^e\|cze}{ec|ze}ig;  # ec|ze|ma
+
+    s{^ex\|tra(?=$vogal)}{ex|tra|}ig; # ex|tra|or|di|na|ri|a|men|te
+
+    s{u\|i$}{ui}ig; # institui ; chui ; 
+    s{\|?i([ao])$}{i|$1}ig; # jia clarabia
+    s{ei([oa])(?=$|\-)}{ei|$1}ig; # ma|r-chei|a mei|a-ir|m meio
+
+    s{^a\|([bd])([svqnmgfz])($vogal)}{a$1|$2$3}ig;  # ab|sin|to
+    s{(?<=\|)a\|([bd])([svqnmgfz])($vogal)}{a$1|$2$3}ig;  # re|ab|sor|o
+    s{^a\|(b)(r)(o)(?=\|g)}{a$1|$2$3}ig;  # ab|ro|ga|o
+    s{^a\|([bd])([s])\|?($consoante)}{a$1$2|$3}ig;  # abs|trac|to
+    s{^o\|([bd])([svqnmgfz])($vogal)}{o$1|$2$3}ig;  # ob|ser|var
+    s{^o\|([bd])([s])\|?($consoante)}{o$1$2|$3}ig;  # obs|tru|ir
+    s{([qg]u)\|([aei])}{$1$2}i;
+    s{([qg]u)\|([o])$}{$1$2}i;  # ambguo; contguo
+
     s{^($consoante)\|}{$1}i;
-    s{m$}{|_nhem}i;
+    #s{m$}{|_nhem}i;      # removido aps reunio do P-Pal a 2010-06-01
+
+    s{tun\|gs}{tungs}ig;  # tungstnio
+
+    s{p\|s$}{ps}i; # excepo de "ps" como breakpair
+    s{(^p|\-p)\|s([i])}{$1s$2}i; # excepo de "ps" como breakpair => m|di|co-psi|co|l|gi|co
+
+    s{s\|s$}{ss}i; # excepo de "ss" como breakpair => fitness
+
+    s{\|\|}{\|}g;
   }
   $p
 }
